# -*- coding: utf-8 -*-
"""
/***************************************************************************
 CtrlCraftDialogChartEstimatedBlocs
                                 A QGIS plugin
 Areva - plugin for displaying 3D mines
                             -------------------
        begin                : 2017-09-12
        git sha              : $Format:%H$
        copyright            : (C) 2017 by Oslandia
        email                : sylvain.beorchia@oslandia.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
"""

from builtins import next
import os
import csv
import configparser

from PyQt5 import uic
from PyQt5.QtWidgets import (QFileDialog,
                         qApp,
                         QComboBox,
                         QHBoxLayout,
                         QLabel,
                         QDialog)

from qgis.gui import QgsMessageBar
from qgis.core import Qgis

from CtrlCraft import CtrlCraftLogger

FORM_CLASS, _ = uic.loadUiType(os.path.join(
    os.path.dirname(__file__), 'ui', 'estimated_blocs_import.ui'))


class CtrlCraftOfficeEstimatedBlocsImport(QDialog, FORM_CLASS):
    """
    Dialog to import Estimated Blocs in database.
    """

    def __init__(self, data, projIni, deposit, parent=None):
        super(CtrlCraftOfficeEstimatedBlocsImport, self).__init__(parent)
        self.setupUi(self)
        self.data = data
        self.projIni = projIni
        self.deposit = deposit
        self.updated = False
        self.config = configparser.ConfigParser()
        self.config.read(self.projIni)
        self.default_radiometry_col = 'eU*_CU-GLOBAL_5x5x1.5_Q{0.00000}'

        self.__populateProjComboBox()
        self.mCancelButton.clicked.connect(self.__close)
        self.mImportButton.clicked.connect(self.__import)
        self.mSelectButton.clicked.connect(self.__select)

        self.mProgressBar.setValue(0)
        self.mLabel.setText('')

    def __close(self):
        self.close()

    def __select(self):
        dlg = QFileDialog()
        dlg.setFileMode(QFileDialog.AnyFile)
        dlg.setNameFilter("CSV files (*.txt *.csv)")

        filenames = []
        if dlg.exec_():
            filenames = dlg.selectedFiles()

        if filenames:
            filename = filenames[0]

            # read header to fill combobox
            with open(filename, 'r') as csvfile:
                reader = csv.reader(csvfile, delimiter=',')
                header = next(reader)

                if self.__is_valid(header):
                    self.mComboBox.addItems(header)

                    idx = self.mComboBox.findText(self.default_radiometry_col)
                    if idx:
                        self.mComboBox.setCurrentIndex(idx)

                    self.mLineEdit.setText(filename)
                else:
                    CtrlCraftLogger().logBar('Invalid format', Qgis.Critical)

    def __populateProjComboBox(self):
        for deposit in self.config:
            self.mProjComboBox.addItem(deposit)
        self.mProjComboBox.setCurrentIndex(self.mProjComboBox.findText(self.deposit) if self.mProjComboBox.findText(self.deposit) != -1 else 0)
        self.mProjComboBox.setEnabled(False)
        return

    def __import(self):
        filename = self.mLineEdit.text()
        if not filename:
            return
        deposit = self.deposit if self.deposit != None else 'DEFAULT'
        trans_x = float(self.config[deposit]['x']) if deposit != 'DEFAULT' else 0
        trans_y = float(self.config[deposit]['y']) if deposit != 'DEFAULT' else 0
        trans_z = float(self.config[deposit]['z']) if deposit != 'DEFAULT' else 0
        self.mSelectButton.setEnabled(False)
        self.mCancelButton.setEnabled(False)
        self.mImportButton.setEnabled(False)

        rad_col = self.mComboBox.currentText()

        self.mLabel.setText(self.tr('Insert estimated blocs in database...'))
        self.mProgressBar.setValue(0)
        qApp.processEvents()

        nlines = 0
        with open(filename, 'r') as csvfile:
            nlines = len(list(csv.reader(csvfile, delimiter=','))) - 1

        new_blocs = 0
        with open(filename, 'r') as csvfile:
            reader = csv.reader(csvfile, delimiter=',')

            index_radiometry = None
            nline = 0
            for row in reader:
                # header
                if not index_radiometry:
                    try:
                        index_radiometry = row.index(rad_col)
                        continue
                    except ValueError:
                        msg = (self.tr('Invalid column \'{}\' for estimated blocs radiometry')
                               .format(rad_col))
                        return

                # subset of columns
                sample_number = row[0]
                x = float(row[1]) + trans_x
                y = float(row[2]) + trans_y
                z = float(row[3]) + trans_z

                rad = row[index_radiometry]
                if rad == 'na':
                    rad = -999

                # insert into database
                if not self.data.estimated_bloc(sample_number):
                    self.data.estimated_bloc_create(sample_number, rad, x, y, z)
                    new_blocs += 1

                # Update progress bar
                nline += 1
                total = nline*100/nlines
                self.mProgressBar.setValue(total)

            self.mProgressBar.setValue(100)

            self.mLabel.setText(self.tr('Create index...'))
            self.mProgressBar.setValue(0)
            self.mProgressBar.setMinimum(0)
            self.mProgressBar.setMaximum(0)
            qApp.processEvents()
            self.data.estimated_bloc_create_index()

            self.mLabel.setText(self.tr('Statistics calculation...'))
            qApp.processEvents()
            self.data.estimated_bloc_stats()

            msg = (self.tr('{} estimated blocs imported with success ({} ignored)!')
                   .format(new_blocs, nlines-new_blocs))
            self.mLabel.setText(msg)

            self.mProgressBar.setMaximum(100)
            self.mProgressBar.setValue(100)

            self.updated = True

        self.mCancelButton.setEnabled(True)

    def __is_valid(self, header):
        valid_header = [
            'SN+ Sample Number (READONLY)+',
            'X+ Coordinate (READONLY)+',
            'Y+ Coordinate (READONLY)+',
            'Z+ Coordinate (READONLY)+',
            'eU*_CU-GLOBAL_5x5x1.5_M{0.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{0.30000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{0.40000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{0.50000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{0.60000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{0.70000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{0.80000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{0.90000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{1.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{1.10000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{1.20000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{1.30000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{1.40000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{1.50000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{1.60000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{1.70000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{1.80000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{1.90000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{2.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{3.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{4.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{6.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_M{8.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{0.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{0.30000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{0.40000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{0.50000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{0.60000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{0.70000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{0.80000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{0.90000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{1.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{1.10000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{1.20000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{1.30000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{1.40000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{1.50000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{1.60000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{1.70000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{1.80000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{1.90000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{2.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{3.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{4.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{6.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_Q{8.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{0.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{0.30000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{0.40000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{0.50000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{0.60000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{0.70000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{0.80000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{0.90000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{1.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{1.10000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{1.20000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{1.30000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{1.40000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{1.50000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{1.60000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{1.70000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{1.80000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{1.90000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{2.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{3.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{4.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{6.00000}',
            'eU*_CU-GLOBAL_5x5x1.5_T{8.00000}',
            'enveloppes_ARTOIS',
            'proportions_raw_20170320{U1}',
            'proportions_raw_20170320{U2}',
            'proportions_raw_20170320{U4+U3}',
            'RESSOURCES']

        return header == valid_header
